; this program builds a file for the
; dot program to generate a plot of a binary
; arithmetic expression built with the constructors
; and accessors in expression.scm

; this program requires that the selected
; language be MzScheme due to the output

(load "expression.scm")

; output-port defaults to (current-output-port) if
; not supplied 

(define (plot-expression t . output-port)
  
  (define (build-plot oport)
    
    ; build the graph header for tree t
    
    (define (build-header)
      (display "digraph g {" oport)
      (newline oport)
      (display "label=\"" oport)
      (display t oport)
      (display "\"" oport)
      (newline oport)
      (display "center=true" oport)
      (newline oport))
    
    (define (plot-operator name value)
      (display (string-append name " [color=black,shape=ellipse,label=\"") oport)
      (display value oport)
      (display "\"]" oport)
      (newline oport))
    
    (define (plot-operand name value)
      (display (string-append name " [color=black,shape=ellipse,label=\"") oport)
      (display value oport)
      (display "\"]" oport)
      (newline oport))
    
    (define (plot-operator-links name)
      (begin
        (display (string-append "\"" name "\" -> " 
                                (string-append name "l")) oport)
        (newline oport))
      
      (begin
        (display (string-append "\"" name "\" -> "
                                (string-append name "r")) oport)
        (newline oport)))
    
    ; build the main body of the graph for tree t
    
    (define (build-body)
      (define (traverse-tree nodename exp)
        (plot-operator nodename (operator exp))
        (if (pair? (left exp))
            (traverse-tree (string-append nodename "l") (left exp))
            (plot-operand (string-append nodename "l") (left exp)))
        (if (pair? (right exp))
            (traverse-tree (string-append nodename "r") (right exp))
            (plot-operand (string-append nodename "r") (right exp)))
        (plot-operator-links nodename))
      (if (and (not (null? t)) (pair? t))
          (traverse-tree "root" t)))
    
    ; build the graph footer for tree t
    
    (define (build-footer)
      (display "}" oport)
      (newline oport))
    
    ; this is the body of build-plot
    ; call each of the sections in order
    
    (build-header)
    (build-body)
    (build-footer))
  
  ; this is the body of plot-expression
  ; build the input file for dot
  
  (if (null? output-port)
      (build-plot (current-output-port))
      (build-plot (car output-port))))
 